@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import unittest
+import json
 from tests import *
 
 
@@ -191,3 +192,46 @@ class TestPipelinesDefaultOrg(unittest.TestCase):
             self.assertTrue(True, "Description added to Pipeline")
         except Exception as e:
             self.assertTrue(False, e)
+
+    def test_wait_for_pipeline(self):
+        """
+        Verify waitfor pipeline endpoint with default org
+        :return:
+        """
+        try:
+            _ = self.pipeline.wait_for_pipeline(
+                pipeline_id=test_pipeline["pipeline"]["id"]
+            )
+
+            self.assertTrue(True, "Pipeline became ready")
+        except Exception as e:
+            self.assertTrue(False, e)
+
+    def test_group_components(self):
+        original_name = "Group components {}".format(str(uuid.uuid4())[:10])
+        new_pipe = self.pipeline.create_pipeline(
+            name=original_name,
+            function=False,
+            org_id=test_org
+        )
+
+        self.pipeline.wait_for_pipeline(
+            org_id=test_org,
+            pipeline_id=new_pipe["pipeline"]["id"]
+        )
+
+        self.pipeline.update_pipeline_from_json(
+            update_payload=json.loads(open("tests/data/pipeline.json", "r+").read()),
+            pipeline_name=original_name,
+            pipeline_id=new_pipe["pipeline"]["id"],
+            org_id=test_org,
+        )
+
+        try:
+            response = self.pipeline.group_components(
+                pipeline_id=new_pipe["pipeline"]["id"],
+                components=["44564", "674714"]
+            )
+            self.assertTrue(True, response)
+        except Exception as e:
+            self.assertTrue(False, e)
