@@ -12,7 +12,8 @@ class DatasetGrapql(GraphQLBuilder):
             page_num=1,
             page_size=100,
             is_deleted=False,
-            facets=None
+            facets=None,
+            other_filters: dict = None
     ):
 
         art_count = self.build_generic_query(
@@ -29,6 +30,7 @@ class DatasetGrapql(GraphQLBuilder):
         fields = [
             "table",
             "numPipelines",
+            "latestSequenceNumber",
             self.build_simple_query(
                 query_name="labels",
                 fields=[
@@ -70,6 +72,10 @@ class DatasetGrapql(GraphQLBuilder):
             for facet in facets:
                 filters += ' && facets = "{}"'.format(str(facet))
 
+        if other_filters:
+            for key_filter, value_filter in other_filters.items():
+                filters += ' && ({} ~ \"{}\")'.format(key_filter, value_filter)
+
         results = {
             "operationName": "fetchDatasets",
             "query": final_query,
