@@ -102,6 +102,13 @@ class Sources(ApiRequestManager):
 
             self.gql_builder = DatasetGrapql()
 
+            # Datastreams
+            self.datastream_events_info_url = self.props.get_remote(
+                self.cluster) + "datasets/v1/dataset/{}/events"
+
+            self.datastream_download_events = self.props.get_remote(
+                self.cluster) + "datasets/v1/dataset/{}/events/download"
+
         except Exception as error:
             raise error
 
@@ -1088,6 +1095,32 @@ class Sources(ApiRequestManager):
 
         raise Exception(repr(get_sources_request.content))
 
+    def get_sources_by_filters(self, org_id, filters):
+        """
+        Retieves all datasets that matches given filters.
+
+        :param org_id: Org id
+        :type org_id: int or str
+        :param filters: Dictonary with filter by name and value. Example: {'name':dataset_name}
+        :type filters: dict of key value pair
+        :return:
+        """
+
+        query = self.gql_builder.build_dataset_query(
+            other_filters=filters
+        )
+        get_sources_request = self.graph_ql_query(org_id=org_id, query=query)
+
+        if get_sources_request.status_code in range(200, 203):
+            data = json.loads(get_sources_request.content)
+            if "data" in data:
+                if "artifacts" in data["data"]:
+                    return data["data"]["artifacts"]
+
+            return []
+
+        raise Exception(repr(get_sources_request.content))
+
     @single_org
     def delete_source(self, org_id=None, *, source_id):
         """
@@ -1117,8 +1150,11 @@ class Sources(ApiRequestManager):
 
         if delete_request.status_code in range(200, 203):
             return json.loads(delete_request.content)
-
-        raise Exception(json.loads(delete_request.content))
+        try:
+            msg = json.loads(delete_request.content)
+            raise Exception(msg)
+        except json.JSONDecodeError as e:
+            raise Exception(str(delete_request.content))
 
     @single_org
     def preview_dataset(self, org_id=None, *, dataset_id, page_num=1):
@@ -1148,7 +1184,11 @@ class Sources(ApiRequestManager):
         if get_response.status_code in [200, 201]:
             return json.loads(get_response.content)
 
-        raise Exception(json.loads(get_response.content))
+        try:
+            msg = json.loads(get_response.content)
+            raise Exception(msg)
+        except json.JSONDecodeError:
+            raise Exception(str(get_response.content))
 
     @single_org
     def get_dataset_info(self, org_id=None, *, dataset_id):
@@ -1177,7 +1217,11 @@ class Sources(ApiRequestManager):
         if get_response.status_code in [200, 201]:
             return json.loads(get_response.content)
 
-        raise Exception(json.loads(get_response.content))
+        try:
+            msg = json.loads(get_response.content)
+            raise Exception(msg)
+        except json.JSONDecodeError:
+            raise Exception(str(get_response.content))
 
     @single_org
     def upload_spreadsheet(self, name=None, file_path=None, org_id=None, sheet_name=None):
@@ -1239,3 +1283,110 @@ class Sources(ApiRequestManager):
             artifact_id=dataset_id,
             artifact_type="dataset"
         )
+
+    @single_org
+    def get_datastream_events_info(self, org_id=None, *, dataset_id, sequenceNr=0):
+        """
+        Returns a list with all events occurred in a datastream
+
+        :param org_id: Optional, Organization id where the dataset was created
+        :param dataset_id: dataset id number
+        :param sequenceNr: Optional, the number of the first event to be returned
+        :return: A list with the events
+        """
+        header = self.build_header(org_id=org_id)
+        params = {'fromSequenceNr': sequenceNr}
+        response = self.session.get(
+            self.datastream_events_info_url.format(dataset_id),
+            params=params,
+            headers=header
+        )
+
+        if response.status_code in self.OK:
+            result = response.json()
+            return result['events']
+
+        try:
+            msg = json.loads(response.content)
+            raise Exception(msg)
+        except json.JSONDecodeError:
+            raise Exception(str(response.content))
+
+    @single_org
+    def get_datastream_event_data(self, org_id=None, *, dataset_id, sequenceNr):
+        """
+        Returns the data of the datastream event specified in the 'sequenceNr' param
+
+        :param org_id: Optional, Organization id where the dataset was created
+        :param dataset_id: dataset id number
+        :param sequenceNr: the number of the datastream event to be returned
+        :return: A csv string with the data starting from event specified
+        """
+        header = self.build_header(org_id=org_id)
+        params = {'fromSequenceNr': sequenceNr}
+        response = self.session.get(
+            self.datastream_download_events.format(dataset_id),
+            params=params,
+            headers=header
+        )
+
+        if response.status_code in self.OK:
+            return response.text
+
+        try:
+            msg = json.loads(response.content)
+            raise Exception(msg)
+        except json.JSONDecodeError:
+            raise Exception(str(response.content))
+
+    @single_org
+    def create_command(self, org_id=None, *,
+                       schema: dict,
+                       name: str,
+                       facets: list,
+                       description: str,
+                       cause: list,
+                       dataset_uuid=None):
+        """
+
+        :param org_id: organization id
+        :type org_id: str
+        :param schema: dataset schema
+        :type schema: dict
+        :param name: datatset name
+        :type name: str
+        :param facets: labels to be include don the dataset
+        :type facets: list
+        :param description: Description of the dataset
+        :type description: str
+        :param cause: The reason for this command
+        :type cause: list
+        :param dataset_uuid: Override of the dataset id
+        :type dataset_uuid: str
+        :return: server response from the server
+        """
+
+        body = {
+            "commands": [{
+                "$type": "create",
+                "schema": schema,
+                "org": str(org_id),
+                "name": name,
+                "facets": facets,
+                "description": description,
+                "cause": cause
+            }]
+        }
+
+        data_set_id = dataset_uuid if dataset_uuid is not None else str(uuid.uuid4())
+
+        try:
+
+            response = self._commit_dataset_command(org_id=org_id, data_set_id=data_set_id, command=body)
+
+            response["uuid"] = data_set_id
+
+            return response
+
+        except Exception as es:
+            raise Exception(es)
